import os
from flask import Blueprint, request, jsonify,session
# from itsdangerous import TimedJSONWebSignatureSerializer as Serializer
from werkzeug.security import generate_password_hash,check_password_hash
from sqlalchemy import or_,func
from authlib.jose import jwt, JoseError
from model.Model import User,Role,Organization,Menu,Permissions,Element
from manager.serverCenter import server
from manager.tools import sqlOrmToJson
import time

db=server.db

user = Blueprint("users", __name__, url_prefix='/user')

SECRET_KEY = 'abcdefghijklmm'

import hashlib
 

# 生成token
def generate_auth_token(user_id, expiration=36000):
    header = {'alg': 'HS256'}
    data={'user_code': user_id}
    return jwt.encode(header=header,payload=data,key=SECRET_KEY)


@user.route('/login', methods=["POST"])
def login():
    '''
    {
        "name":用户名
        "password":密码
    }
    '''
    data = request.json
    user = db.session.query(User).filter_by(name=data.get("name")).first()
    if not user:
        db.session.close()
        return jsonify({"code": 501, "message": "用户不存在", "data": ""}),501
    elif user.disable==1:
        db.session.close()
        return jsonify({"code": 502, "message": "账号被禁用"}),502
    elif not check_password_hash(user.password,data.get("password")):
        if user.try_time<5:
            user.try_time +=1
        else:
            user.disable=1
        db.session.commit()
        disableCount=5-user.try_time
        db.session.close()
        return jsonify({"code": 405, "message": f"密码错误，请重试({disableCount}次后账号将被锁定)", "data": ""}),405
    else:
        user.token = generate_auth_token(user.id).decode()
        user.try_time = 0
        db.session.commit()
        session["id"] = user.id
        session["name"] = user.name
        session['token'] = str(user.token)
        datas = {'id': user.id, "name": user.name, "token": user.token}
        db.session.commit()
        db.session.close()
        return jsonify({"code": 200, "message": "请求成功", "data": datas}),200

@user.route("/logout",methods=["POST"])
def logout():
    session.clear()
    return jsonify({"code": 200, "message": "请求成功"}),200

@user.route("/adduser",methods=["POST"])
def adduser():
    '''
    {
        name 
        password
        role_id
        organization_id 组织id 非必传
        disable 非必传
        eamil 非必传
    }
    '''
    
    data=request.json
    if not data.get("name"):
        return jsonify({"code": 401, "message": "用户名不能为空"}),401
    if not data.get("password"):
        return jsonify({"code": 401, "message": "密码不能为空"}),401
    if not data.get("role_id"):
        return jsonify({"code": 401, "message": "角色不能为空"}),401
    if db.session.query(User).filter_by(name=data.get("name")).count()>1:
        db.session.close()
        return jsonify({"code": 401, "message": "用户名重复，请重试"}),401
    create_time=time.strftime('%Y-%m-%d %H:%M:%S',time.localtime(time.time()))
    if db.session.query(User).filter_by(name=data.get("name")).count()>0:
        return jsonify({"code": 503, "message": "该用户已添加"}),503
    db.session.add(User(name=data.get("name"),password=generate_password_hash(data.get("password")),role_id=int(data.get("role_id")),organization_id=(data.get("organization_id") or None),create_date=create_time,create_user=(session.get("id") or 0),
                        disable=(data.get("disable") or 0),jira_account=data.get("jira_account"),jira_password=data.get("jira_password"),email_address=(data.get("email_address") or None)))
    db.session.commit()
    db.session.close()
    return jsonify({"code": 200, "message": "添加成功"}),200
    
@user.route("/getuser/<id_>",methods=["GET"])
def getuser(id_):
    res=sqlOrmToJson(db.session.query(User.id,User.name,User.role_id,User.organization_id,User.create_date,User.jira_account,User.jira_password,User.create_user,User.email_address,User.email_password).filter_by(id=int(id_)).first())
    db.session.close()
    return jsonify({"code": 200, "message": "请求成功","data":res}),200
    

@user.route("/updateuser",methods=["POST"])
def updateuser():
    '''
        id 
        password 非必传 改密码时可调用
    '''
    data=request.json
    user_id=data.get("id")
    if (user_id==session.get("id")):
        if data.get("password"):
            oldPass=generate_password_hash(data.get("oldpassword"))
            if oldPass!=db.session.query(User.password).filter_by(id=user_id).first().password:
                db.session.close
                return jsonify({"code": 503, "message": "旧密码错误"}),503
            data["password"]=generate_password_hash(data.get("password"))
            del data['secondpassword']
            del data['oldpassword']
    try:
        del data["create_date"]
    except:
        pass
    try:
        userinfo=db.session.query(User).filter_by(id=user_id)
        userinfo.update(data)
        userinfo.try_time=0
        db.session.commit()
        db.session.close()
        return jsonify({"code": 200, "message": "修改成功"})
    except:
        db.session.rollback()
        db.session.close()
        return jsonify({"code": 401, "message": "修改失败"}),401
    

@user.route("/user_list",methods=["GET"])
def finduser():
    '''
        page_size 页数
        page_num 当前页
        name
        organizations 数组
        roles 数组
    '''
    data=request.args
    page_size = int(data.get("page_size"))
    page_num = int(data.get("page_num"))
    users=db.session.query(User.id,User.name,User.disable,func.date_format(User.create_date, "%Y-%m-%d %H:%i:%s").label('create_date'),User.role_id,Role.name.label("role_name"),Organization.id.label("organization_id"),Organization.path).outerjoin(Role,Role.id==User.role_id).outerjoin(Organization,Organization.id==User.organization_id)
    if data.get("name"):
        users=users.filter(User.name.like("%"+data.get("name")+"%"))
    if data.getlist('organizations[]'):
        info=[]
        for i in data.getlist('organizations[]'):
            info.append(Organization.find_path.like("%-"+i+"-%"))
        users=users.filter(or_(*info))
    if data.getlist("roles[]"):
        users=users.filter(Role.id.in_(data.getlist("roles[]")))
    result=users.paginate(page=page_num, per_page=page_size)
    total_sum = result.total
    res = result.items
    db.session.close()
    return jsonify({"code": 200, "message": "请求成功", "data": {"rows": sqlOrmToJson(res), "total": total_sum}})

@user.route("/userdetail",methods=["GET"])
def userdetail():
    userinfo=db.session.query(User.id,User.name,User.email_address,Role.name.label("role_name"),Organization.path.label("organization")).filter(User.role_id==Role.id,User.organization_id==Organization.id,User.id==session.get("id")).first()
    print(userinfo.email_address)
    usermenu=db.session.query(Menu.id,Menu.name,Menu.router,Menu.icon,Menu.parent_id,Menu.path).filter(Permissions.type==0,Permissions.p_id==Menu.id,Permissions.role_id==Role.id,User.role_id==Role.id,User.id==session.get("id")).all()
    usermenu=sorted(usermenu,key=lambda x:len(list(filter(None,x.path.split("/")))),reverse=False)
    result=[]
    for each in usermenu:
        target=result
        info=list(filter(None,each.path.split("/")))
        for path in range(len(info)):
            breakinfo=False
            for index in range(len(target)):
                if target[index]["id"]==int(info[path]):
                    target=target[index]["children"]
                    breakinfo=True
                    break
            if not breakinfo and path==len(info)-1:
                target.append({"name":each.name,"router":each.router,"id":each.id,"icon":each.icon,"children":[]})
            elif breakinfo:
                continue
            else:
                break
        target=result
    db.session.close()
    res=sqlOrmToJson(userinfo)
    res["menu"]=result
    return jsonify({"code": 200, "message": "请求成功", "data": res})

@user.route("/elements",methods=["GET"])
def elements():
    id_=request.args.get("id")
    res=db.session.query(Element.u_id).filter(User.id==session.get("id"),Role.id==User.role_id,Permissions.role_id==Role.id,Permissions.type==1,Permissions.p_id==Element.id,Element.menu_id==int(id_)).all()
    result=[i.u_id for i in res]
    db.session.close()
    return jsonify({"code": 200, "message": "请求成功", "data": result})



