import sys
import os
project_path=os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.append(project_path)
from manager.serverCenter import server
from sqlalchemy import Column,Integer,String,Integer,String,DateTime,text,Text
db=server.db

class Menu(db.Model):
    '''
    菜单表
    '''
    __tablename__="menu"
    __table_args__ = {'mysql_engine' : 'InnoDB', 'mysql_charset' : 'utf8mb4'}
    id=Column(Integer, primary_key=True,autoincrement=True)
    name=Column(String(64),  nullable=False,comment='菜单名')
    router=Column(String(64),  nullable=True,comment='路由')
    icon=Column(String(64),  nullable=True,comment='菜单图标')
    parent_id=Column(Integer, nullable=False,server_default=text("0"),comment='父菜单id。为0时表示顶级菜单')
    path=Column(String(64),  nullable=True,comment='寻址路径')
    default_show=Column(Integer, nullable=False,server_default=text("0"),comment='默认所有角色均有该权限,1为是')
    
class Element(db.Model):
    '''
    菜单有权限的元素表
    '''
    __tablename__="element"
    __table_args__ = {'mysql_engine' : 'InnoDB', 'mysql_charset' : 'utf8mb4'}
    id=Column(Integer, primary_key=True,autoincrement=True)
    menu_id=Column(Integer, nullable=False,comment='父菜单id')
    u_id=Column(String(64), nullable=True,comment='元素uid,由前端提供')
    name=Column(String(64),  nullable=False,comment='元素名')
    default_show=Column(Integer, nullable=False,server_default=text("0"),comment='默认是否展示,为0时表示不展示')

class Organization(db.Model):
    '''
    组表
    '''
    __tablename__="organization"
    __table_args__ = {'mysql_engine' : 'InnoDB', 'mysql_charset' : 'utf8mb4'}
    id=Column(Integer, primary_key=True,autoincrement=True)
    name=Column(String(64),  nullable=False,comment='组名')
    parent_id=Column(Integer, nullable=False,server_default=text("0"),comment='父组织的id。为0时表示顶级菜单')
    find_path=Column(String(64),  nullable=False,comment='寻找路径')
    path=Column(String(64),  nullable=False,comment='组织全路径名')

class Role(db.Model):
    '''
    角色表
    '''
    __tablename__="role"
    __table_args__ = {'mysql_engine' : 'InnoDB', 'mysql_charset' : 'utf8mb4'}
    id=Column(Integer, primary_key=True,autoincrement=True)
    name=Column(String(64),  nullable=False,comment='角色名')
    
    
class User(db.Model):
    '''
    用户表
    '''
    __tablename__="user"
    __table_args__ = {'mysql_engine' : 'InnoDB', 'mysql_charset' : 'utf8mb4'}
    id=Column(Integer, primary_key=True,autoincrement=True)
    name=Column(String(64),  nullable=False,comment='用户名')
    role_id=Column(Integer,  nullable=False,comment='角色id')
    organization_id=Column(Integer, nullable=True,comment='组织id')
    create_date=Column(DateTime,nullable=True,comment="创建时间")
    password=Column(Text(64),  nullable=False,comment='密码')
    disable=Column(Integer, nullable=False,server_default=text("0"),comment='受否禁用')
    jira_account=Column(String(64),  nullable=True,comment='jira登录账户')
    jira_password=Column(String(64),  nullable=True,comment='jira登录密码')
    email_address=Column(String(64), nullable=True,comment='邮箱地址')
    email_password=Column(String(64), nullable=True,comment='邮箱密码')
    create_user=Column(Integer, nullable=False,comment='创建人')
    try_time=Column(Integer, nullable=False,server_default=text("0"),comment='重试次数')
    token=Column(Text(64),  nullable=True,comment='登录token')

class Permissions(db.Model):
    '''
    权限表
    '''
    __tablename__="permissions"
    __table_args__ = {'mysql_engine' : 'InnoDB', 'mysql_charset' : 'utf8mb4'}
    id=Column(Integer, primary_key=True,autoincrement=True)
    role_id=Column(String(64),  nullable=False,comment='角色id')
    type=Column(Integer, nullable=False,server_default=text("0"),comment='0为菜单权限,1为元素权限')
    p_id=Column(Integer, nullable=False,comment='对应权限得id,0时为菜单得id,1时为元素的id')
    status=Column(Integer, nullable=False,server_default=text("0"),comment='1true-0false')
    update_date=Column(DateTime,nullable=False,comment="更新时间")
    update_user=Column(Integer, nullable=False,comment='更新人')
    
class Emails(db.Model):
    '''
    角色表
    '''
    __tablename__="emails"
    __table_args__ = {'mysql_engine' : 'InnoDB', 'mysql_charset' : 'utf8mb4'}
    id=Column(Integer, primary_key=True,autoincrement=True)
    name=Column(String(64),  nullable=False,comment='邮件的用户名')
    default_cc=Column(Integer, nullable=False,server_default=text("0"),comment='默认抄送人,1true-0false')
    address=Column(String(64),  nullable=False,comment='地址')
    
class Uploadecase(db.Model):
    '''
    上传用例信息记录表
    '''
    __tablename__="uploadcase"
    __table_args__ = {'mysql_engine' : 'InnoDB', 'mysql_charset' : 'utf8mb4'}
    id=Column(Integer, primary_key=True,autoincrement=True)
    filename=Column(String(64),  nullable=True,comment='上传的文件名')
    project=Column(String(64),  nullable=False,comment='项目名')
    user_id=Column(Integer,  nullable=False,comment='上传用例的人员id')
    step=Column(Integer,  nullable=False,server_default=text("0"),comment='当前阶段 0-上传文件 1-插入用例 2-关联用例')
    stepstatus=Column(Integer,  nullable=False,comment='当前步骤的状态 0运行中，1错误，2结束')
    status=Column(Integer,  nullable=False,comment='状态 0-执行中 1-已结束')
    case_count=Column(Integer,  nullable=False,server_default=text("0"),comment='状态 0-执行中 1-已结束')
    create_time=Column(DateTime,nullable=False,comment="创建时间")

if __name__=="__main__":
    with server.app.app_context():
        db.create_all()